/*
 * @(#)CompositeContext.java	1.15 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.awt;

import java.awt.image.Raster;
import java.awt.image.WritableRaster;

/**
 * This interface defines the encapsulated and optimized environment for
 * a compositing operation.  CompositeContext objects maintain state for
 * compositing operations.  In a multi-threaded environment, several
 * contexts may exist simultaneously for a single Composite object.
 * @see Composite
 * @version 10 Feb 1997
 */

public interface CompositeContext {
    /**
     * Release resources allocated for context.
     */
    public void dispose();

    /**
     * This method composes the two source rasters
     * and places the result in the destination raster. Note that
     * the destination can be the same object as either
     * the first or second source.
     * @param src1 The first source for the compositing operation.
     * @param src2 The second source for the compositing operation.
     * @param dst The tile where the result of the operation is stored.
     */
    public void compose(Raster src1,
                        Raster src2,
			WritableRaster dst);


}

