/*
 * @(#)EventQueueListener.java	1.2 98/03/18
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.awt.event;

import java.util.EventListener;
import java.awt.AWTEvent;

/**
 * The listener interface for receiving notification of events
 * posted to the EventQueue.  Unlike the other EventListeners
 * in this package, EventQueueListeners passively observe
 * events being posted to the queue.  Most applications should
 * never use this class; applications which might include event
 * recorders for automated testing, and facilities such as
 * the Java Accessibility package.
 * <p>
 * The class that is interested in monitoring EventQueue events
 * implements this interface, and the object created with that
 * class is registered with the EventQueue, using the EventQueue's
 * <code>addEventQueueListener</code> method. When an event is
 * posted to the EventQueue, that object's <code>eventPosted</code> 
 * method is invoked.
 *
 * @see AWTEvent
 *
 * @version 1.2 03/18/98
 * @author Thomas Ball
 */
public interface EventQueueListener extends EventListener {

    /**
     * Invoked when an event is posted to an EventQueue.
     */
    public void eventPosted(AWTEvent e);

}
