/*
 * @(#)RasterImageConsumer.java	1.5 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.awt.image;

import java.util.Hashtable;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageProducer;
import java.awt.image.ColorModel;
/**
 * The interface for objects expressing interest in image data through
 * the ImageProducer interfaces.  When a consumer is added to an image
 * producer, the producer delivers all of the data about the image
 * using the method calls defined in this interface.
 *
 * @see ImageProducer
 *
 * @version 10 Feb 1997
 */
public interface RasterImageConsumer extends ImageConsumer {
    /**
     * The pixels of the image are delivered using one or more calls
     * to the setPixels method.  x and y specify the location and
     * Raster.getWidth() and Raster.getHeight() specify the size
     * of the rectangle of source pixels that are contained in the
     * Raster. The specified ColorModel object should
     * be used to convert the pixels into their corresponding color
     * and alpha components. Pixels in the Raster can be accessed by
     * using the <a href=java.awt.image.Raster.html#getPixelData>getPixelData</a> method:
     * <pre>
     *      int[] pixel = (int[])Raster.getPixelData(tx, ty, null);
     * </pre>
     * The variable pixel will contain one channel element for the specified
     * location from each channel in the Raster.
     * @see ColorModel
     * @see Raster
     */
     void setPixels(int x, int y, ColorModel model, WritableRaster raster);

}



