/*
 * @(#)JPEGParam.java	1.4 98/03/18
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/* ********************************************************************
 **********************************************************************
 **********************************************************************
 *** COPYRIGHT (c) Eastman Kodak Company, 1997                      ***
 *** As  an unpublished  work pursuant to Title 17 of the United    ***
 *** States Code.  All rights reserved.                             ***
 **********************************************************************
 **********************************************************************
 **********************************************************************/

package java.awt.image.codec;

/** 
 * JPEGParam encapsulates tables and options necessary to control
 * encoding and decoding JPEG images. Parameters are either set
 * explicitly by the application for encoding, or read from the JPEG
 * header for decoding.
 * <p>
 * The selection of color ids for the encoded 
 * and decoded images is used to control the JPEG codec's processing of
 * the data (e.g. chroma band subsampling and color rotation) and does 
 * not necessarily reflect the actual color space of the image data.
 * <p>
 * Note:  The color ids described herein are simply enumerated values that
 * influence data processing by the JPEG codec.  JPEG compression is
 * by definition color blind.  These values are used as hints when compressing 
 * or decompressing JPEG data.
 * The JPEG codec can perform some default rotation of data into 
 * spaces that will aid in getting better compression ratios.
 * <p>
 * In order to control the "color space" behavior of the JPEG related classes,
 * developers have the option of accepting the default codec behavior
 * or "forcing" some types of behavior.
 * <p>
 * The default behavior is defined in the following tables.
 * 
<pre>
	DECODING:
	
	JPEG (Encoded) Color ID			Image (Decoded) Color ID
	=======================			========================
	COLOR_ID_UNKNOWN			COLOR_ID_UNKNOWN
	COLOR_ID_GRAYSCALE			COLOR_ID_GRAYSCALE
	COLOR_ID_RGB				COLOR_ID_RGB
	COLOR_ID_YCbCr				COLOR_ID_RGB
	COLOR_ID_CMYK				COLOR_ID_CMYK
	COLOR_ID_PYCC				COLOR_ID_PYCC
	COLOR_ID_RGBA				COLOR_ID_RGBA
	COLOR_ID_YCbCrA				COLOR_ID_RGBA
	COLOR_ID_RGBA_INVERTED			COLOR_ID_RGBA_INVERTED
	COLOR_ID_YCbCrA_INVERTED		COLOR_ID_RGBA_INVERTED
	COLOR_ID_PYCCA				COLOR_ID_PYCCA
	COLOR_ID_YCCK				COLOR_ID_YCCK
	
	
	ENCODING:
	
	Image (Decoded) Color ID		JPEG (Encoded) Color ID
	========================		=======================
	COLOR_ID_UNKNOWN			COLOR_ID_UNKNOWN
	COLOR_ID_GRAYSCALE			COLOR_ID_GRAYSCALE
	COLOR_ID_RGB				COLOR_ID_YCbCr
	COLOR_ID_YCbCr				COLOR_ID_YCbCr
	COLOR_ID_CMYK				COLOR_ID_CMYK
	COLOR_ID_PYCC				COLOR_ID_PYCC
	COLOR_ID_RGBA				COLOR_ID_YCbCrA
	COLOR_ID_YCbCrA				COLOR_ID_YCbCrA
	COLOR_ID_RGBA_INVERTED			COLOR_ID_YCbCrA_INVERTED
	COLOR_ID_YCbCrA_INVERTED		COLOR_ID_YCbCrA_INVERTED
	COLOR_ID_PYCCA				COLOR_ID_PYCCA
	COLOR_ID_YCCK				COLOR_ID_YCCK
	
</pre>
 * When the user wants behavior other that what is specified in the tables
 * above the setForceColorConversion() method must be invoked passing a
 * value of true as a parameter.  When this is the case the user generally
 * wants to avoid a default color conversion or wants to control the color
 * id used in processing the data.  If the JPEG codec cannot convert the data
 * using the specified values of ImageColorID and EncodedColorID, an
 * ImageFormatException will be thrown.  The allowable conversions are listed
 * in the tables below:
<pre>
	DECODING:
	
	JPEG (Encoded) Color ID			Image (Decoded) Color ID
	=======================			========================
	COLOR_ID_UNKNOWN			COLOR_ID_UNKNOWN
	COLOR_ID_GRAYSCALE			COLOR_ID_GRAYSCALE
	COLOR_ID_YCbCr				COLOR_ID_GRAYSCALE
	COLOR_ID_RGB				COLOR_ID_RGB
	COLOR_ID_YCbCr				COLOR_ID_RGB
	COLOR_ID_CMYK				COLOR_ID_CMYK
	COLOR_ID_YCCK				COLOR_ID_CMYK
	COLOR_ID_PYCC				COLOR_ID_PYCC
	COLOR_ID_RGBA				COLOR_ID_RGBA
	COLOR_ID_YCbCrA				COLOR_ID_RGBA
	COLOR_ID_YCbCrA				COLOR_ID_YCbCrA
	COLOR_ID_YCbCrA_INVERTED		COLOR_ID_RGBA_INVERTED
	COLOR_ID_YCbCrA_INVERTED		COLOR_ID_YCbCrA_INVERTED
	COLOR_ID_PYCCA				COLOR_ID_PYCCA
	COLOR_ID_YCCK				COLOR_ID_YCCK
	
	
	ENCODING:
	
	Image (Decoded) Color ID		JPEG (Encoded) Color ID
	========================		=======================
	COLOR_ID_UNKNOWN			COLOR_ID_UNKNOWN
	COLOR_ID_GRAYSCALE			COLOR_ID_GRAYSCALE
	COLOR_ID_RGB				COLOR_ID_GRAYSCALE
	COLOR_ID_YCbCr				COLOR_ID_GRAYSCALE
	COLOR_ID_RGB				COLOR_ID_RGB
	COLOR_ID_RGB				COLOR_ID_YCbCr
	COLOR_ID_YCbCr				COLOR_ID_YCbCr
	COLOR_ID_CMYK				COLOR_ID_CMYK
	COLOR_ID_PYCC				COLOR_ID_PYCC
	COLOR_ID_RGBA				COLOR_ID_RGBA
	COLOR_ID_RGBA				COLOR_ID_YCbCrA
	COLOR_ID_YCbCrA				COLOR_ID_YCbCrA
	COLOR_ID_RGBA_INVERTED			COLOR_ID_YCbCrA_INVERTED
	COLOR_ID_CMYK				COLOR_ID_YCCK
        COLOR_ID_PYCCA				COLOR_ID_PYCCA
	COLOR_ID_YCCK				COLOR_ID_YCCK
	
</pre>
 * When encoding JPEG streams, if the image color ID is not one of
 * COLOR_ID_UNKNOWN, COLOR_ID_GRAYSCALE, COLOR_ID_RGB, COLOR_ID_YCbCr,
 * COLOR_ID_CMYK, or COLOR_ID_YCCK, then the value of writeJFIFHeaders
 * should be set to false.  This is due to idiosyncracies that exist
 * between the JPEG specification and JFIF.
 */

public class JPEGParam {
  
    // Known Color IDs

    /** Unknown or Undefined Color ID */
    public final static int COLOR_ID_UNKNOWN = 0;
    
    /** Monochrome */
    public final static int COLOR_ID_GRAYSCALE = 1;

    /** Red, Green, and Blue */
    public final static int COLOR_ID_RGB = 2;

    /** YCbCr */
    public final static int COLOR_ID_YCbCr = 3;

    /** CMYK */
    public final static int COLOR_ID_CMYK = 4;

    /** PhotoYCC */
    public final static int COLOR_ID_PYCC = 5;
 
    /** RGB-Alpha */
     public final static int COLOR_ID_RGBA = 6;
 
    /** YCbCr-Alpha */
    public final static int COLOR_ID_YCbCrA = 7;

    /** RGB-Alpha with R, G, and B inverted.*/
    public final static int COLOR_ID_RGBA_INVERTED = 8;
 
    /** YCbCr-Alpha with Y, Cb, and Cr inverted.*/
    public final static int COLOR_ID_YCbCrA_INVERTED = 9;

    /** PhotoYCC-Alpha */
    public final static int COLOR_ID_PYCCA = 10;

    /** YCbCrK */
    public final static int COLOR_ID_YCCK = 11;	    


    /** Maximium color id value allowed. */
    private static int _JCSMAX = 12;
    /** Number of components for each Color id */
    private static byte[] numComponents = new byte[_JCSMAX];

    static {
	numComponents[COLOR_ID_GRAYSCALE] = 1;
	numComponents[COLOR_ID_RGB] = 3;
	numComponents[COLOR_ID_YCbCr]=3;
	numComponents[COLOR_ID_CMYK]=4;
	numComponents[COLOR_ID_PYCC]=3;
 	numComponents[COLOR_ID_RGBA]=4;
	numComponents[COLOR_ID_YCbCrA]=4;
	numComponents[COLOR_ID_RGBA_INVERTED]=4;
	numComponents[COLOR_ID_YCbCrA_INVERTED]= 4;
	numComponents[COLOR_ID_PYCCA]= 4;
	numComponents[COLOR_ID_YCCK]= 4;
    }
	/** Slow but accurate integer algorithm */
	public final static int JDCT_ISLOW=0;
	/** Faster, less accurate integer algorithm */
	public final static int JDCT_IFAST=1;
	/** Floating point algorithm - accurate but speed is hardware dependent */
	public final static int JDCT_FLOAT=2;
	/** The default algorithm */
	protected int JDCT_DEFAULT=JDCT_ISLOW;
	/** Number of allowed HuffmanTables */
    public final static int  NUM_HUFF_TBLS = 4;  
    /** DC Huffman table type */
	public final static int  DC_TABLE_TYPE = 0;
	/** AC Huffman table types */
	public final static int  AC_TABLE_TYPE = 1;
	/** Color id of the (decoded) image */
	private int imageColorID;
	/** Color id of the encoded data */
	private int encodedColorID;	
	/** Width of encoded image as read from JFIF header */
	private int width;
	/** Height of encoded image as read from JFIF header */
	private int height;
	/** DCT Algorithm */
	private int DCTMethod;
	/** Quantization tables 1..4 */
	private JPEGQTable qtables[];
	/** DC Entropy encoding tables 1..4 */
	private JPEGHuffmanTable dc_hufftables[];
	/** AC Entropy encoding tables 1..4 */
	private JPEGHuffmanTable ac_hufftables[];
	/** Optimize entropy encoding tables */
	private boolean optimizeCoding;

	/** Write the JFIF header (when encoding) */
	private boolean writeJFIFHeader;
	/** Write the Adobe marker (when encoding) */
	private boolean writeAdobeMarker;
	/** Write the tables to the output stream 
	 * Allows for encoding image only streams
	 * Default is true
	 */
	private boolean writeTables;
	/** Write the image data to the output stream 
	 * Allows for encoding tables only streams
	 * Default is true
	 */
	private boolean writeImage;
	/** 1..100, or 0 for no input smoothing	*/
	private int smoothingFactor;
	/** MCUs per restart, or 0 for no restart */
	private int restartInterval;			
	/** if > 0, MCU rows per restart interval */
	private int restartInRows;				
	/** gamma of the input image */
	private double inputGamma;
	/** image gamma wanted in output  */
	private double outputGamma;		

	/** These 3 values are copied into the JFIF APP0 marker. 
	 * They are not used by the JPEG code */

	/** JFIF code for pixel size units */
	private byte densityUnit;
	/** Horizontal pixel density */
	private int xDensity;
	/** Vertical pixel density */
    private int yDensity;
    /** Flag that determines the behavior of the JPEG lib color conversion */
	private boolean forceColorConversion;

	
	/** Constructs a JPEGParam with default options.
	 * The compression tables are set to the standard quantization 
	 * and encoding tables for 8 bit data with a linear quality level
	 * of 75.
	 */
	public JPEGParam() {
		qtables = new JPEGQTable[4];
		ac_hufftables = new JPEGHuffmanTable[4];
		dc_hufftables = new JPEGHuffmanTable[4];
		for( int i = 0; i < 4 ; i++ ) {
			qtables[i] = null ;
			dc_hufftables[i] = null ;
			ac_hufftables[i] =  null ;
		}
		setDefaults();
	}

	/** Sets the Huffman encoding tables to those in JPEG standard section K.3
	 * These are only valid for 8-bit data precision.
	 */
	protected void stdHuffmanTables() {
		JPEGHuffmanTable        tempACLum, tempACChrom, tempDCLum, tempDCChrom;
	
		tempDCLum = new JPEGHuffmanTable(JPEGHuffmanTable.lengths_dc_luminance,
										 JPEGHuffmanTable.symbols_dc_luminance);
		setHuffmanTable(tempDCLum, 0,  DC_TABLE_TYPE );

		tempACLum = new JPEGHuffmanTable(JPEGHuffmanTable.lengths_ac_luminance,
										 JPEGHuffmanTable.symbols_ac_luminance);
		setHuffmanTable(tempACLum, 0,  AC_TABLE_TYPE);

		tempDCChrom = new JPEGHuffmanTable(JPEGHuffmanTable.lengths_dc_chrominance,
										   JPEGHuffmanTable.symbols_dc_chrominance);
		setHuffmanTable(tempDCChrom, 1,  DC_TABLE_TYPE);

		tempACChrom = new JPEGHuffmanTable(JPEGHuffmanTable.lengths_ac_chrominance,
										   JPEGHuffmanTable.symbols_ac_chrominance);
		setHuffmanTable(tempACChrom, 1, AC_TABLE_TYPE);

	}

	/** Constructs JPEG quantization tables appropriate for the
	 * indicated quality setting (0..100). If force_baseline is TRUE,
	 * the quantization table entries are constrained to the range
	 * 1..255 for full JPEG baseline compatibility. The base quantization
	 * table is the sample given in the JPEG spec section K.1
	 * @param quality - the quality setting ranging from 0 - 100
	 * @param force_baseline - boolean value that force baseline quantization.
	 *                         see JPEG spec for details
	 */
	public void setQuality( int quality, boolean force_baseline ) {

		int q = quality;
		JPEGQTable     tempQTblPtr;
		/* Convert user 0-100 rating to percentage scaling */
		if ( q <=0 ) q=1;
		if ( q> 100) q=100;

		/* The basic table is used as-is (scaling 100) for a quality of 50.
		 * Qualities 50..100 are converted to a scaling percentage 200-2*Q
		 * Note that at Q=100 the scaling is 0, which will cause all of the
		 * table entries to be 1 (hence, no quantization loss).
		 * Qualities 1..50 are converted to scaling percentage 5000/Q
		 */
		if (q < 50 )
			q = 5000/q;
		else
			q = 200 - q*2;
		/* Set up standard quality tables */
		tempQTblPtr = new JPEGQTable( JPEGQTable.std_luminance_quant_tbl );
		tempQTblPtr.scaleTable( quality, force_baseline );
		setQTable(tempQTblPtr, 0 );

		tempQTblPtr = new JPEGQTable( JPEGQTable.std_chrominance_quant_tbl );
		tempQTblPtr.scaleTable( quality, force_baseline );
		setQTable(tempQTblPtr, 1 );
	}

	
	/** Sets all parameters to reasonable defaults. Equivalent to the
	 * jpeg_set_defaults() helper routine in the IJG library, with
	 * the exception that the encoded and decoded color id is set
	 * to COLOR_ID_RGB instead of being read from the JFIF header.
	 */

    protected  void	setDefaults() {
		writeJFIFHeader=true;
		writeTables = true;
		writeImage = true;
		optimizeCoding=false;
		writeAdobeMarker=false;
		smoothingFactor=0;
		DCTMethod = JDCT_DEFAULT;
		densityUnit=0;
		xDensity=1;
		yDensity=1;
		restartInterval=0;
		restartInRows=0;
		outputGamma = 1.0;
		inputGamma = 1.0;
		encodedColorID = COLOR_ID_YCbCr ;
		imageColorID = COLOR_ID_RGB ;
		forceColorConversion = false;
		setQuality( 75, true );
		stdHuffmanTables();
	}


	/** Returns the coefficient quantization tables or NULL if not defined 
	 * @param int tableNum - the index of the table to be returned.
	 * @return JPEGQTable - Quantization table stored at index tableNum.
	 * @Exception IllegalArgumentException - thrown if the table index is out
	 * of range.  Index must range in value from 0 - 3.
	 */
	public JPEGQTable  getQTable(int tableNum ) {
		if (tableNum < 0 || tableNum > 3 )  
			{ throw new IllegalArgumentException( "tableNum must be between 0 and 3."); }
		return qtables[tableNum];
	}


	/** Sets the coefficient quantization tables at index passed. 
	 * @param  JPEGQTable - QTable that will be assigned to index tableNum.
	 * @param int tableNum - the index of the table to be set.
	 * @return void.
	 * @Exception IllegalArgumentException - thrown if the table index is out
	 * of range.  Index must range in value from 0 - 3.
	 */	
	public void	setQTable( JPEGQTable qtable, int tableNum ) {
		if (tableNum < 0 || tableNum > 3 )  
			{ throw new IllegalArgumentException( "tableNum must be between 0 and 3."); }
		qtables[tableNum] = qtable;
	}

	/** Returns the  Huffman coding table or NULL if not defined 
	 * @param int tableNum - the index of the table to be returned.
	 * @param int type 0=DC table, 1=AC table
	 * @return JPEGHuffmanTable - Huffman table stored at index tableNum.
	 * @Exception IllegalArgumentException - thrown if the table index is out
	 * of range.  Index must range in value from 0 - 3.  Also thrown if the table
	 * type being requested is not 0 or 1.
	 */	
	public JPEGHuffmanTable getHuffmanTable( int tableNum, int type ){
  		if (tableNum < 0 || tableNum > NUM_HUFF_TBLS )  
			{ throw new IllegalArgumentException( "tableNum must be between 0 and 4."); }
		if ( type ==  AC_TABLE_TYPE ) 
			if( ac_hufftables[tableNum] != null )
				return (JPEGHuffmanTable)ac_hufftables[tableNum].clone();
			else
				return null;
		else if (type == DC_TABLE_TYPE )
			if( dc_hufftables[tableNum] != null )
				return (JPEGHuffmanTable)dc_hufftables[tableNum].clone();
			else
				return null;
		else
			{ throw new IllegalArgumentException( "type must be 0 or 1."); }
	}


	/** Sets the Huffman coding table at index passed. 
	 * @param  JPEGHuffmanTable - JPEGHuffmanTable that will be assigned to index tableNum.
	 * @param int tableNum - the index of the table to be set.
	 * @ param type 0=DC table, 1=AC table
	 * @return void.
	 * @Exception IllegalArgumentException - thrown if the table index is out
	 * of range.  Index must range in value from 0 - 3. Also thrown if the type being
	 * set is not equal to 0 or 1.
	 */		
	public void	setHuffmanTable( JPEGHuffmanTable hufftable, int tableNum, int type ) {
		if (tableNum < 0 || tableNum > NUM_HUFF_TBLS )  
			{ throw new IllegalArgumentException( "tableNum must be between 0 and 4."); }
		if ( type == AC_TABLE_TYPE )
			ac_hufftables[tableNum] = (JPEGHuffmanTable)hufftable.clone();
		else if (type ==  DC_TABLE_TYPE )
			dc_hufftables[tableNum] = (JPEGHuffmanTable)hufftable.clone();
		
		else
			{ throw new IllegalArgumentException( "type must be 0 or 1."); }
	}


	/** Selects the DCT algorithm
	 * @param dctAlg - algorithm type.  Must be one of the following values
	 *  JDCT_ISLOW,  JDCT_IFAST,  JDCT_IFLOAT.  The default value is  JDCT_ISLOW.
	 */
	public void	setDCTMethod( int dctAlg) {
		DCTMethod = dctAlg;
	}
	/** Retrievs the DCT algorithm
	 * @return int - current setting for the DCT algorithm type.
	 */
	public int	getDCTMethod( ) {return DCTMethod;	}


	/** Set flag indicating whether to optimize entropy encoding tables 
	 * @param boolean flag - indicates whether to set this option in the jpeg codec.
	 * the default value is false.
	 */
	public void setoptimizeCoding( boolean flag){ optimizeCoding = flag;}

	/** Get flag indicating whether to optimize entropy encoding tables
	 * @return  boolean - indicating wheter the optimize coding flag was set
	 * in the codec.  Applies to encoding only.
	 */
	public boolean getoptimizeCoding( ) { return optimizeCoding; }

	/** Set flag indicating whether to write the JFIF header (when encoding) 
	 * @param boolean flag - tells the jpeg compressor whether to write the JFIF
	 * header into the data stream.  The default value is true.
         * Applies to encoding only.
	 */
	public void setwriteJFIFHeader( boolean flag ){ writeJFIFHeader = flag; }

	/** Get flag indicating whether to write the JFIF header when encoding 
	 * @return boolean - indicating the current state of the writeJFIFHeader flag
	 * that will be applied to any data streams that are written.
	 */
	public boolean getwriteJFIFHeader() { return writeJFIFHeader; }

	/** Set flag indicating whether to write the Adobe marker (when encoding)
	 * @param boolean flag - tells the jpeg compressor whether to write the Adobe 
	 * markers into the data stream.
	 */
	public void setwriteAdobeMarker( boolean flag ){writeAdobeMarker = flag; }

	/** Get flag indicating whether to write the Adobe marker when encoding
	 * @return boolean - a value indicating whether Adobe markers will be written
	 * the the jpeg data stream.
	 */
	public boolean getwriteAdobeMarker() { return writeAdobeMarker; }

	/** Set flag indicating whether to write the tables when encoding 
	   * @param boolean - a value directing the JPEG compressor to write the
	   * JPEG tables into the jpeg data stream.  
	   */
	public void setwriteTables( boolean flag ) { writeTables = flag; }

	/** Get flag indicating whether to write the tables when encoding. 
	 * @return boolean - returns the current jpeg compressor setting that indicates
	 * whether the tables will be written to the jpeg data stream.
	 */
	public boolean getwriteTables() { return writeTables; }

	/** Set flag indicating whether to write the image data when encoding jpeg
	 * data streams.
	 * @param boolean - directs the jpeg compressor to include the image data into
	 * subsequent data streams.
	 */
	public void setwriteImage( boolean flag ) { writeImage = flag; }

	/** Get flag indicating whether to write the image data when encoding 
	 * @return boolean - value indicatine the current disposition of this parameter
	 * in the codec.
	 */
	public boolean getwriteImage() { return writeImage; }

	/** Set the input smoothing factor parameter applied when encoding.
	 * @param int  factor 1..100, or 0 for no input smoothing
	 */
	public void setsmoothingFactor( int factor ) { 
		if ( factor > 0 && factor <=100 )
			{ smoothingFactor = factor; }
		else { throw new IllegalArgumentException("The valid range for smoothing factor is 1..100"); }
	}

	/** Get the input smoothing factor 
	 * @return int - returns the value of the smoothing factor currently set in the jpeg codec for encoding.
	 */
	public int getsmoothingFactor() { return smoothingFactor; }

	/** Set the MCUs per restart, or 0 for no restart 
	 * @param int - returns the restart MCU setting for the jpeg codec.
	 */
	public void setrestartInterval( int restartMCUs ) { restartInterval = restartMCUs; }

	/** Get the MCUs per restart
	 * @return int - the number of MCUs applied to the jpeg codec when restarting.
	 */
	public int getrestartInterval() { return restartInterval; }

	/** Set MCU rows per restart interval
	 * @param int - restart rows that are applied when the jpeg codec restarts.
	 */
	public void setrestartInRows( int restartRows ) { restartInRows = restartRows; }

	/** Get MCU rows per restart interval
	 * @return int - the current jpeg codec setting for the value of restart in rows.
	 */
	public int getrestartInRows( ) { return restartInRows; }

	/** Set the gamma of the input image 
	 * @param double - the gamma value of the input image.
	 */
	public void setinputGamma( double gamma) { inputGamma = gamma;}

	/** Get the gamma of the input image 
	 * @return double - the gamma value of the input image.
	 */
	public double getinputGamma( ) { return inputGamma; }

	/** Set the image gamma wanted in output 
	 * @param double - gamma of the output image
	 */
	public void setoutputGamma( double gamma ) { outputGamma = gamma;}

	/** Get the image gamma wanted in the output
	 * @return double - the image gamma desired in the output.
	 */
	public double getoutputGamma( ) { return outputGamma; }

	/** Set the JFIF code for pixel size units 
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	 * @param byte  0=unknown, 1=dots/inch, 2=dots/cm.
	 */
	public void setdensityUnit( byte unit) { densityUnit = unit; }

	/** Get the JFIF code for pixel size units 
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	 * @return byte - value indicating the density unit  0=unknown, 1=dots/inch, 2=dots/cm.
	 */
	public byte getdensityUnit() { return densityUnit; }

	/** Set the horizontal pixel density
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	 * @param int - horizontal pixel density.
	 */
	public void setxDensity( int density ) { xDensity = density; }

	/** Get the horizontal pixel density 
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	  * @return int - the horizontal pixel density.
	  */
	public int getxDensity() { return xDensity; }

	/** Set the vertical pixel density.
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	  * @param int - the verticle pixel density
	  */
	public void setyDensity( int density ) { yDensity = density; }

	/** Get the vertical pixel density 
	 * This value is copied into the JFIF APP0 marker. It isn't used by the JPEG code.
	  * @return int - the verticle pixel density.
	  */
	public int getyDensity() { return yDensity; }

	/** Get the image width
	 * @return int the width of the image data passed throught the codec in pixels.
	 */
	public int  getWidth(  ) {return width; }

	/** Set the image width
	 * @param int - the width of the image data that is to be encoded.
	 */
	public void setWidth( int w ) {width = w; }

	/** Get the image height
	 * @return int - height of the image data passed throught the codec in pixels.
	 */
	public int  getHeight() {return height ; }  

	/** Set the image height
	 * @param int - the height of the image data being encoded in pixels.
	 */
	public void setHeight( int h) {height = h ; }  

	/** Sets the value of the decoded color id.
	 * @param int - integer value of the decoded color id.  When encoding image data
	 * this must correspond to the color id of the image data.  When decompressing image
	 * data this corresponds to the color id of the data being returned.  
	 * @exception IllegalArgumentException - thrown when the value of the of the color id
	 * that is passed is not a valid value as described in the publicly defined color id
	 * values.
	 */
    public void setImageColorID( int id ) {
        if ( id < 0 || id > _JCSMAX )
			{  throw new IllegalArgumentException("Invalid JPEG Color ID."); }
		imageColorID = id;
    }
	/** Sets the value of the Encoded color id
	 * @param int - the integer value of the color id of the data after it has been 
	 * encoded.  This is used as a hint to the JPEG codec.  There are no guarantees that
	 * that color conversion as part of data compression will be performed.
	 * @exception  IllegalArgumentException - thrown when the value of the of the color id
	 * that is passed is not a valid value as described in the publicly defined color id
	 * values.
	 */
    public void setEncodedColorID( int id ) {
        if ( id < 0 || id > _JCSMAX )
			{  throw new IllegalArgumentException("Invalid JPEG Color ID."); }
		encodedColorID = id;
    }
    /** Returns the JPEG Image color id
	 * @return int - returns the current value of the decoded color id.
	 */
    public int getImageColorID() { return imageColorID;}

    /** Returns the JPEG Encoded color id
	 * @return int - returns the value of the JPEG encoded color id.
	 */
    public int getEncodedColorID() { return encodedColorID;}

	/** Returns the number of components for the color id passed as a parameter.
	 * @param id - a color id.
	 * @return int - the number of components
	 * @exception  IllegalArgumentException - thrown when the value of the of the color id
	 * that is passed is not a valid value as described in the publicly defined color id
	 * values.
	 */
	public int getNumComponents(int id)	{
        if ( id < 0 || id > _JCSMAX )
			{  throw new IllegalArgumentException("Invalid JPEGColorID."); }
		else
			return numComponents[id];	
	}
	/** Returns the value of the forceColorConversion data member.
	 * @return boolean - if false the default coversions will happen as defined in the class
	 * header comments.  If true, JPEG lib will be instructed to decode the image data into the
	 * DecodedColorID.  If there is no conversion routine available an ImageFormatException 
	 * will be thrown indicating an invalid color conversion.
	 */
	 public boolean getForceColorConversion(){ return forceColorConversion;}
	/** Sets the value of the forceColorConversion data member.
	 * @param boolean - if false the default coversions will happen as defined in the class
	 * header comments.  If true, JPEG lib will be instructed to decode the image data into the
	 * DecodedColorID.  If there is no conversion routine available an ImageFormatException 
	 * will be thrown indicating an invalid color conversion.
	 */
    public void setForceColorConversion(boolean flag) {
	forceColorConversion = flag;
    }

}
