/*
 * @(#)RenderedImageFactory.java	1.4 98/03/18
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/* ********************************************************************
 **********************************************************************
 **********************************************************************
 *** COPYRIGHT (c) Eastman Kodak Company, 1997                      ***
 *** As  an unpublished  work pursuant to Title 17 of the United    ***
 *** States Code.  All rights reserved.                             ***
 **********************************************************************
 **********************************************************************
 **********************************************************************/

package java.awt.image.renderable;
import java.awt.image.RenderedImage;
import java.util.Hashtable;

/**
 * The RenderedImageFactory interface (often abbreviated RIF) is
 * intended to be implemented by classes that wish to act as factories
 * to produce different renderings (as a chain of BufferedImageOps or
 * similar datatypes), depending on a specific set of parameters,
 * properties, and rendering hints.  For example, a
 * RenderedImageFactory could determine whether a hardware-specific
 * operation is appropriate for a particular task.
 */
public interface RenderedImageFactory {

  /**
   * Creates a RenderedImage representing the results of an imaging
   * operation (or chain of operations) for a given ParameterBlock and
   * hints.  The RIF may also query any source images referenced by
   * the ParameterBlock for their dimensions, SampleModels,
   * properties, etc., as necessary.
   *
   * <p> The create() method can return null if the
   * RenderedImageFactory is not capable of producing output for the
   * given set of source images and parameters.  For example, if a
   * RenderedImageFactory is only capable of performing a 3x3
   * convolution on single-banded image data, and the source image has
   * multiple bands or the convolution Kernel is 5x5, null should be
   * returned.  Hints should be taken into account, but can be
   * ignored.
   *  
   * @param pb a ParameterBlock containing sources and parameters
   *        for the RenderedImage to be created.
   * @param renderHints a Hashtable of hints to the RenderedImageFactory.
   * @param renderHintsObserved a Hashtable of hints observed by the RIF.
   * @return a RenderedImage containing the desired output image.
   */
  RenderedImage create(ParameterBlock pb, 
                       Hashtable renderHints, 
                       Hashtable renderHintsObserved); 
}
