/*
 * @(#)HyperlinkEvent.java	1.3 98/02/02
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package java.awt.swing.event;

import java.util.EventObject;
import java.net.URL;


/**
 * HyperlinkEvent is used to notify interested parties that 
 * something has happened with respect to a hypertext link.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.3 02/02/98
 * @author  Timothy Prinzing
 */
public class HyperlinkEvent extends EventObject {

    public HyperlinkEvent(Object source, EventType type, URL u) {
        super(source);
	this.type = type;
	this.u = u;
    }

    /**
     * The type of event.
     */
    public EventType getEventType() {
	return type;
    }

    /**
     * The URL that the link refers to.
     */
    public URL getURL() {
	return u;
    }

    private EventType type;
    private URL u;

	
    public static final class EventType {

        private EventType(String s) {
	    typeString = s;
	}

	public static final EventType ENTERED = new EventType("ENTERED");
	public static final EventType EXITED = new EventType("EXITED");
	public static final EventType ACTIVATED = new EventType("ACTIVATED");

        public String toString() {
	    return typeString;
	}

	private String typeString;
    }
}

