/*
 * @(#)MetalButtonBorder.java	1.10 98/02/03
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
 
package java.awt.swing.plaf.metal;

import java.awt.swing.*;
import java.awt.swing.border.*;
import java.awt.swing.plaf.basic.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.swing.plaf.*;

/**
 * MetalButtonBorder implementation
 *  This is the border that "normal" buttons use.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.10 02/03/98
 * @author Steve Wilson
 */
public class MetalButtonBorder extends AbstractBorder {

    protected static Insets borderInsets = new Insets( 3, 3, 3, 3 );
  
    public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
        JButton button = (JButton)c;
	ButtonModel model = button.getModel();

	if ( model.isEnabled() ) {
	    if ( model.isPressed() && model.isArmed() ) {
	        MetalUtils.drawPressed3DBorder( g, x, y, w, h );
	    }
	    else {
	        if (button.isDefaultButton()) {
		  MetalUtils.drawDefaultButtonBorder( g, x, y, w, h, button.hasFocus() && button.isFocusPainted() );
		} else {
		MetalUtils.drawButtonBorder( g, x, y, w, h, button.hasFocus() && button.isFocusPainted());
		}
	    }
	} else { // disabled state
	    MetalUtils.drawDisabledBorder( g, x, y, w-1, h-1 );
	}
    }

    public Insets getBorderInsets( Component c ) {
        return borderInsets;
    }
}

