/*
 * @(#)MotifLookAndFeel.java	1.75 98/02/04
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package java.awt.swing.plaf.motif;


import java.awt.Color;
import java.awt.Font;

import java.awt.swing.*;
import java.awt.swing.plaf.*;
import java.awt.swing.border.*;

import java.awt.swing.plaf.basic.BasicLookAndFeel;
import java.awt.swing.plaf.basic.BasicMarginBorder;
import java.awt.swing.plaf.basic.BasicComboBoxRenderer;
import java.awt.swing.plaf.basic.BasicComboBoxEditor;

/**
 * Implements the Motif Look and Feel.
 * UI classes not implemented specifically for Motif will
 * default to those implemented in Basic.  
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @version 1.75 02/04/98
 * @author unknown
 */
public class MotifLookAndFeel extends BasicLookAndFeel
{
    public String getName() {
	return "CDE/Motif";
    }

    public String getID() {
	return "Motif";
    }

    public String getDescription() {
	return "The CDE/Motif Look and Feel";
    }


    public boolean isNativeLookAndFeel() {
	String osName = System.getProperty("os.name");
	return (osName != null) && (osName.indexOf("Solaris") != -1);
    }


    public boolean isSupportedLookAndFeel() {
	return true;
    }


    /**
     * Load the SystemColors into the defaults table.  The keys
     * for SystemColor defaults are the same as the names of
     * the public fields in SystemColor.  If the table is being
     * created on a native Motif platform we use the SystemColor
     * values, otherwise we create color objects whose values match
     * the default CDE/Motif colors.
     */
    protected void initSystemColorDefaults(UIDefaults table)
    {
	String[] systemColors = {
   	          "desktop", "#005C5C", /* Color of the desktop background */
    	    "activeCaption", "#000080", /* Color for captions (title bars) when they are active. */
        "activeCaptionText", "#FFFFFF", /* Text color for text in captions (title bars). */
      "activeCaptionBorder", "#B24D7A", /* Border color for caption (title bar) window borders. */
          "inactiveCaption", "#AEB2C3", /* Color for captions (title bars) when not active. */
      "inactiveCaptionText", "#000000", /* Text color for text in inactive captions (title bars). */
    "inactiveCaptionBorder", "#AEB2C3", /* Border color for inactive caption (title bar) window borders. */
	           "window", "#AEB2C3", /* Default color for the interior of windows */
             "windowBorder", "#AEB2C3", /* ??? */
	       "windowText", "#000000", /* ??? */
		     "menu", "#AEB2C3", /* ??? */
	         "menuText", "#000000", /* ??? */
		     "text", "#FFF7E9", /* Text background color */
	         "textText", "#000000", /* Text foreground color */
	    "textHighlight", "#000000", /* Text background color when selected */
        "textHighlightText", "#FFF7E9", /* Text color when selected */
         "textInactiveText", "#808080", /* Text color when disabled */
	          "control", "#AEB2C3", /* Default color for controls (buttons, sliders, etc) */
	      "controlText", "#000000", /* Default color for text in controls */
         "controlHighlight", "#DCDEE5", /* Highlight color for controls */
       "controlLtHighlight", "#DCDEE5", /* Light highlight color for controls */
	    "controlShadow", "#63656F", /* Shadow color for controls */
       "controlLightShadow", "#9397A5", /* Shadow color for controls */		  
          "controlDkShadow", "#000000", /* Dark shadow color for controls */
	        "scrollbar", "#AEB2C3", /* Scrollbar ??? color. PENDING(jeff) foreground? background? ?*/
		     "info", "#FFF7E9", /* ??? */
	         "infoText", "#000000"  /* ??? */
	};

	loadSystemColors(table, systemColors);
    }


    protected void initClassDefaults(UIDefaults table)
    {
	super.initClassDefaults(table);

	String motifPackageName = "java.awt.swing.plaf.motif.";
	Object[] uiDefaults = {
		   "ButtonUI", motifPackageName + "MotifButtonUI",
		 "CheckBoxUI", motifPackageName + "MotifCheckBoxUI",
 	    "DirectoryPaneUI", motifPackageName + "MotifDirectoryPaneUI",
 	      "FileChooserUI", motifPackageName + "MotifFileChooserUI",
                    "LabelUI", motifPackageName + "MotifLabelUI",
		  "MenuBarUI", motifPackageName + "MotifMenuBarUI",
		     "MenuUI", motifPackageName + "MotifMenuUI",
		 "MenuItemUI", motifPackageName + "MotifMenuItemUI",
	 "CheckBoxMenuItemUI", motifPackageName + "MotifCheckBoxMenuItemUI",
      "RadioButtonMenuItemUI", motifPackageName + "MotifRadioButtonMenuItemUI",
	      "RadioButtonUI", motifPackageName + "MotifRadioButtonUI",
	     "ToggleButtonUI", motifPackageName + "MotifToggleButtonUI",
		"PopupMenuUI", motifPackageName + "MotifPopupMenuUI",
	      "ProgressBarUI", motifPackageName + "MotifProgressBarUI",
		"ScrollBarUI", motifPackageName + "MotifScrollBarUI",
    	       "ScrollPaneUI", motifPackageName + "MotifScrollPaneUI",
		   "SliderUI", motifPackageName + "MotifSliderUI",
	        "SplitPaneUI", motifPackageName + "MotifSplitPaneUI",
	       "TabbedPaneUI", motifPackageName + "MotifTabbedPaneUI",
		 "TextAreaUI", motifPackageName + "MotifTextAreaUI",
		"TextFieldUI", motifPackageName + "MotifTextFieldUI",
	    "PasswordFieldUI", motifPackageName + "MotifPasswordFieldUI",
		 "TextPaneUI", motifPackageName + "MotifTextPaneUI",
	       "EditorPaneUI", motifPackageName + "MotifEditorPaneUI",
		     "TreeUI", motifPackageName + "MotifTreeUI",
	    "InternalFrameUI", motifPackageName + "MotifInternalFrameUI",
	      "DesktopPaneUI", motifPackageName + "MotifDesktopPaneUI",
	        "SeparatorUI", motifPackageName + "MotifSeparatorUI",
	        "OptionPaneUI", motifPackageName + "MotifOptionPaneUI",
     	         "ComboBoxUI", motifPackageName + "MotifComboBoxUI",
	      "DesktopIconUI", motifPackageName + "MotifDesktopIconUI",
	};

	table.putDefaults(uiDefaults);
    }

    protected void initComponentDefaults(UIDefaults table) 
    {
	super.initComponentDefaults(table);

	FontUIResource dialogPlain12 = new FontUIResource("Dialog", 
                                                          Font.PLAIN, 12);
	FontUIResource serifPlain12 = new FontUIResource("Serif", 
                                                          Font.PLAIN, 12);
	FontUIResource sansSerifPlain12 = new FontUIResource("SansSerif", 
                                                          Font.PLAIN, 12);
	FontUIResource monospacedPlain12 = new FontUIResource("Monospaced", 
                                                          Font.PLAIN, 12);
	ColorUIResource red = new ColorUIResource(Color.red);
	ColorUIResource black = new ColorUIResource(Color.black);
        ColorUIResource white = new ColorUIResource(Color.white);
	ColorUIResource lightGray = new ColorUIResource(Color.lightGray);
	ColorUIResource controlDarker = new ColorUIResource(147, 151, 165);  // slate blue
	ColorUIResource scrollBarTrack = controlDarker;
	ColorUIResource menuItemPressedBackground = new ColorUIResource(165,165,165);
	ColorUIResource menuItemPressedForeground = new ColorUIResource(0,0,0);


	Object loweredBevelBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifBorderFactory.getLoweredBevelBorder();
	    }
	};

	Object focusBevelBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return new BorderUIResource( new CompoundBorder(
		    MotifBorderFactory.getFocusBorder(),
		    MotifBorderFactory.getLoweredBevelBorder()) );
	    }
	};

	Object raisedBevelBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifBorderFactory.getRaisedBevelBorder();
	    }
	};

	Object buttonBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return new BorderUIResource( new CompoundBorder(
		    MotifBorderFactory.getButtonBorder(),
		    BasicMarginBorder.getMarginBorder()) );
	    }
	}; 
	
	Object buttonToggleBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return new BorderUIResource( new CompoundBorder(
		    MotifBorderFactory.getToggleButtonBorder(),
		    BasicMarginBorder.getMarginBorder()) );
	    }
	};

	Object fieldTextBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifBorderFactory.getFieldBorder();
	    }
	};


	Object multilineTextBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return BasicMarginBorder.getMarginBorder();
	    }
	};

	Object menuItemCheckIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifIconFactory.getMenuItemCheckIcon();
	    }
	};

	Object menuItemArrowIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifIconFactory.getMenuItemArrowIcon();
	    }
	};

	Object menuArrowIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifIconFactory.getMenuArrowIcon();
	    }
	};

	Object checkBoxIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifIconFactory.getCheckBoxIcon();
	    }
	};

	Object radioButtonIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifIconFactory.getRadioButtonIcon();
	    }
	};

	Object unselectedTabBackground = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) { 
		Color c = (Color)table.get("control");
                return new ColorUIResource(Math.max((int)(c.getRed()*.85),0), 
                                           Math.max((int)(c.getGreen()*.85),0), 
                                           Math.max((int)(c.getBlue()*.85),0));
	    }
	};

	Object unselectedTabForeground = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) { 
		Color c = (Color)table.get("controlText");
                return new ColorUIResource(Math.max((int)(c.getRed()*.85),0), 
                                           Math.max((int)(c.getGreen()*.85),0), 
                                           Math.max((int)(c.getBlue()*.85),0));
	    }
	};

	Object unselectedTabShadow = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) { 
		Color c = (Color)table.get("control");
                Color base = new Color(Math.max((int)(c.getRed()*.85),0), 
                                       Math.max((int)(c.getGreen()*.85),0), 
                                       Math.max((int)(c.getBlue()*.85),0));
                return new ColorUIResource(base.darker());
	    }
	};

	Object unselectedTabHighlight = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) { 
		Color c = (Color)table.get("control");
                Color base = new Color(Math.max((int)(c.getRed()*.85),0), 
                                       Math.max((int)(c.getGreen()*.85),0), 
                                       Math.max((int)(c.getBlue()*.85),0));
                return new ColorUIResource(base.brighter());
	    }
	};


	// *** Tree

	Object treeOpenIcon = LookAndFeel.makeIcon(getClass(), 
                                                           "icons/TreeOpen.gif");

	Object treeClosedIcon = LookAndFeel.makeIcon(getClass(), 
                                                             "icons/TreeClosed.gif");

	Object treeLeafIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifTreeCellRenderer.loadLeafIcon();
	    }
	};

	Object treeExpandedIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifTreeUI.MotifExpandedIcon.createExpandedIcon();
	    }
	};

	Object treeCollapsedIcon = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return MotifTreeUI.MotifCollapsedIcon.createCollapsedIcon();
	    }
	};


        Object comboBoxRendererActiveValue  = new UIDefaults.ActiveValue() {
            public Object createValue(UIDefaults table) {
                return new MotifComboBoxRenderer.UIResource();
            }
        };

        Object comboBoxEditorActiveValue  = new UIDefaults.ActiveValue() {
            public Object createValue(UIDefaults table) {
                return new BasicComboBoxEditor.UIResource();
            }
        };

	Object menuBarBorder = new UIDefaults.LazyValue(){
	  public Object createValue(UIDefaults table) {
	    return MotifBorderFactory.getMenuBarBorder();
	    // implements UIResource
	  }
	};

	Object menuMarginBorder = new UIDefaults.LazyValue() {
	    public Object createValue(UIDefaults table) {
		return new BorderUIResource(new CompoundBorder(
	               MotifBorderFactory.getLoweredBevelBorder(),
		       BasicMarginBorder.getMarginBorder()) );
	    }
	};

	Object popupMenuBorder = new UIDefaults.LazyValue(){
	  public Object createValue(UIDefaults table) {
	    return new BorderUIResource(MotifBorderFactory.getRaisedBevelBorder());
	  }
	};
	Object focusCellHighlightBorder = new BorderUIResource( 
					      new LineBorder((Color)table.get("activeCaptionBorder")) );

	Object[] defaults = {

	    "Desktop.background", table.get("desktop"),

	    "Panel.background", table.get("control"),
	    "Panel.foreground", table.get("textText"),
	    "Panel.font", dialogPlain12,

	    "ProgressBar.foreground", controlDarker,

	    "Button.border", buttonBorder,
	    "Button.background", table.get("control"),
	    "Button.foreground", table.get("controlText"),
	    "Button.focus", table.get("controlText"),
	    "Button.pressed", table.get("controlLightShadow"),
	    "Button.font", dialogPlain12,

	    "Button.shadow", controlDarker,


	    "RadioButton.background", table.get("control"),
	    "RadioButton.foreground", table.get("controlText"),
	    "RadioButton.font", dialogPlain12,
	    "RadioButton.icon", radioButtonIcon,
	    "RadioButton.focus", table.get("activeCaptionBorder"),
	    "RadioButton.icon", radioButtonIcon,

	    "CheckBox.icon", checkBoxIcon,
	    "CheckBox.focus", table.get("activeCaptionBorder"),

	    "RadioButtonMenuItem.icon", radioButtonIcon,
	    "CheckBoxMenuItem.icon", checkBoxIcon,

	    "ToggleButton.border", buttonToggleBorder,
	    "ToggleButton.background", table.get("control"),
	    "ToggleButton.foreground", table.get("controlText"),
	    "ToggleButton.focus", table.get("controlText"),
	    "ToggleButton.selected",  table.get("controlLightShadow"),
	    "ToggleButton.pressed", table.get("controlLightShadow"),
	    "ToggleButton.font", dialogPlain12,

	    "Menu.border", menuMarginBorder,
	    "Menu.font", dialogPlain12,
	    "Menu.foreground", table.get("menuText"),
	    "Menu.background", table.get("menu"),
	    "Menu.pressedForeground", menuItemPressedForeground,
	    "Menu.pressedBackground", menuItemPressedBackground,
	    "Menu.arrowIcon", menuArrowIcon,

	    "MenuBar.border", menuBarBorder,
	    "MenuBar.background", table.get("menu"),
	    "MenuBar.foreground", table.get("menuText"),
	    "MenuBar.font", dialogPlain12,

	    "MenuItem.border", menuMarginBorder,
	    "MenuItem.font", dialogPlain12,
	    "MenuItem.acceleratorFont", dialogPlain12,
	    "MenuItem.foreground", table.get("menuText"),
	    "MenuItem.background", table.get("menu"),
	    "MenuItem.pressedForeground", menuItemPressedForeground,
	    "MenuItem.pressedBackground", menuItemPressedBackground,
	    "MenuItem.checkIcon", menuItemCheckIcon,
	    "MenuItem.arrowIcon", menuItemArrowIcon,

	    "PopupMenu.background", table.get("menu"),
	    "PopupMenu.border", popupMenuBorder,	    
	    "PopupMenu.foreground", table.get("menuText"),
	    "PopupMenu.font", dialogPlain12,

	    "Label.font", dialogPlain12,
	    "Label.background", table.get("control"),
	    "Label.foreground", table.get("controlText"),

	    "List.focusCellHighlightBorder", focusCellHighlightBorder,

	    "DesktopIcon.icon", LookAndFeel.makeIcon(getClass(), 
                                                             "icons/DesktopIcon.gif"),

	    "ScrollBar.background", scrollBarTrack,
	    "ScrollBar.foreground", table.get("control"),
	    "ScrollBar.track", scrollBarTrack,
	    "ScrollBar.trackHighlight", table.get("controlDkShadow"),
	    "ScrollBar.thumb", table.get("control"),
	    "ScrollBar.thumbHighlight", table.get("controlHighlight"),
	    "ScrollBar.thumbDarkShadow", table.get("controlDkShadow"),
	    "ScrollBar.thumbLightShadow", table.get("controlShadow"),
	    "ScrollBar.border", loweredBevelBorder,
 
	    "ScrollPane.border", null,
	    "ScrollPane.font", dialogPlain12,
	    "ScrollPane.background", table.get("control"),
	    "ScrollPane.foreground", table.get("controlText"),
	    "ScrollPane.viewportBorder", loweredBevelBorder,

	    "Slider.border", focusBevelBorder,
	    "Slider.foreground", table.get("control"),
	    "Slider.background", controlDarker,
	    "Slider.highlight", table.get("controlHighlight"),
	    "Slider.shadow", table.get("controlShadow"),
	    "Slider.focus", table.get("controlDkShadow"),

	    "SplitPane.background", table.get("control"),
	    "SplitPane.highlight", table.get("controlHighlight"),
	    "SplitPane.shadow", table.get("controlShadow"),

            "TabbedPane.font", dialogPlain12,
	    "TabbedPane.tabBackground", table.get("control"),
	    "TabbedPane.tabForeground", table.get("controlText"),
	    "TabbedPane.tabHighlight", table.get("controlHighlight"),
	    "TabbedPane.tabShadow", table.get("controlShadow"),
	    "TabbedPane.unselectedTabBackground", unselectedTabBackground,
	    "TabbedPane.unselectedTabForeground", unselectedTabForeground,
	    "TabbedPane.unselectedTabHighlight", unselectedTabHighlight,
	    "TabbedPane.unselectedTabShadow", unselectedTabShadow,
            "TabbedPane.focus", table.get("activeCaptionBorder"),


	    "Tree.background", controlDarker,                              // default: dark slate blue
	    "Tree.hash", table.get("controlDkShadow"),                     // default: black
	    "Tree.iconShadow", table.get("controlShadow"),
	    "Tree.iconHighlight", table.get("controlHighlight"),
	    "Tree.iconBackground", table.get("control"),                              
	    "Tree.iconForeground", table.get("controlShadow"),             // default: black
	    "Tree.borderSelectionColor", table.get("activeCaptionBorder"), // default: maroon
	    "Tree.textSelectionColor", table.get("textText"),              // default: black
	    "Tree.textNonSelectionColor", table.get("textText"),           // default: black
	    "Tree.backgroundSelectionColor", table.get("text"),            // default: white
	    "Tree.backgroundNonSelectionColor", controlDarker,             // default: dark slate blue
	    "Tree.backgroundNonSelectionColor", controlDarker,             // default: dark slate blue
	    "Tree.openIcon", treeOpenIcon,
	    "Tree.closedIcon", treeClosedIcon,
	    "Tree.leafIcon", treeLeafIcon,
	    "Tree.expandedIcon", treeExpandedIcon,
	    "Tree.collapsedIcon", treeCollapsedIcon,

      	    "Table.focusCellHighlightBorder", focusCellHighlightBorder,
	    "Table.scrollPaneBorder", null,

	    //	    "Table.background", white,  // cell background color
	    //	    "Table.focusCellBackground", white,



            "ComboBox.control", table.get("control"),
            "ComboBox.controlForeground", black,
            "ComboBox.renderer", comboBoxRendererActiveValue,
            "ComboBox.editor",   comboBoxEditorActiveValue,
            "ComboBox.background", table.get("text"), 
            "ComboBox.foreground", black,
            "ComboBox.selectedBackground", black,
            "ComboBox.selectedForeground", table.get("text"),
            "ComboBox.disabledBackground", table.get("control"),
            "ComboBox.disabledForeground", table.get("textInactiveText"),
            "ComboBox.font", dialogPlain12,

	    "TextField.caretForeground", black,
	    "TextField.caretBlinkRate", new Integer(500),
	    "TextField.inactiveForeground", table.get("textInactiveText"),
	    "TextField.selectionBackground", table.get("textHighlight"),
	    "TextField.selectionForeground", table.get("textHighlightText"),
	    "TextField.background", table.get("window"),
	    "TextField.foreground", table.get("textText"),
	    "TextField.font", sansSerifPlain12,
	    "TextField.border", fieldTextBorder,
	    
	    "PasswordField.caretForeground", black,
	    "PasswordField.caretBlinkRate", new Integer(500),
	    "PasswordField.inactiveForeground", table.get("textInactiveText"),
	    "PasswordField.selectionBackground", table.get("textHighlight"),
	    "PasswordField.selectionForeground", table.get("textHighlightText"),
	    "PasswordField.background", table.get("window"),
	    "PasswordField.foreground", table.get("textText"),
	    "PasswordField.font", monospacedPlain12,
	    "PasswordField.border", fieldTextBorder,
	    
	    "TextArea.caretForeground", black,
	    "TextArea.caretBlinkRate", new Integer(500),
	    "TextArea.inactiveForeground", table.get("textInactiveText"),
	    "TextArea.selectionBackground", table.get("textHighlight"),
	    "TextArea.selectionForeground", table.get("textHighlightText"),
	    "TextArea.background", table.get("window"),
	    "TextArea.foreground", table.get("textText"),
	    "TextArea.font", monospacedPlain12,
	    "TextArea.border", multilineTextBorder,
	    
	    "TextPane.caretForeground", black,
	    "TextPane.caretBlinkRate", new Integer(500),
	    "TextPane.inactiveForeground", table.get("textInactiveText"),
	    "TextPane.selectionBackground", lightGray,
	    "TextPane.selectionForeground", table.get("textHighlightText"),
	    "TextPane.background", white,
	    "TextPane.foreground", table.get("textText"),
	    "TextPane.font", serifPlain12,
	    "TextPane.border", multilineTextBorder,
	    
	    "EditorPane.caretForeground", red,
	    "EditorPane.caretBlinkRate", new Integer(500),
	    "EditorPane.inactiveForeground", table.get("textInactiveText"),
	    "EditorPane.selectionBackground", lightGray,
	    "EditorPane.selectionForeground", table.get("textHighlightText"),
	    "EditorPane.background", white,
	    "EditorPane.foreground", table.get("textText"),
	    "EditorPane.font", serifPlain12,
	    "EditorPane.border", multilineTextBorder,

	    "ToolTip.border", raisedBevelBorder,
	    "ToolTip.background", table.get("info"),
	    "ToolTip.foreground", table.get("infoText"),

	    "OptionPane.errorIcon", LookAndFeel.makeIcon(getClass(), 
                                                         "icons/Error.gif"),
	    "OptionPane.informationIcon", LookAndFeel.makeIcon(getClass(), 
                                                               "icons/Inform.gif"),
	    "OptionPane.warningIcon", LookAndFeel.makeIcon(getClass(), 
                                                           "icons/Warn.gif"),
	    "OptionPane.questionIcon", LookAndFeel.makeIcon(getClass(), 
                                                            "icons/Question.gif")
	};

	table.putDefaults(defaults);
    }

}

