/*
 * @(#)MotifTextFieldUI.java	1.14 98/02/02
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package java.awt.swing.plaf.motif;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.swing.*;
import java.awt.swing.plaf.basic.BasicTextFieldUI;
import java.awt.swing.border.*;
import java.awt.swing.event.*;
import java.awt.swing.text.*;
import java.awt.swing.plaf.*;

/**
 * Provides the Motif look and feel for a text field.
 * <p>
 * Warning: serialized objects of this class will not be compatible with
 * future swing releases.  The current serialization support is appropriate
 * for short term storage or RMI between Swing1.0 applications.  It will
 * not be possible to load serialized Swing1.0 objects with future releases
 * of Swing.  The JDK1.2 release of Swing will be the compatibility
 * baseline for the serialized form of Swing objects.
 *
 * @author  Timothy Prinzing
 * @version 1.14 02/02/98
 */
public class MotifTextFieldUI extends MotifTextUI {

    /**
     * Creates a UI for a JTextField.
     *
     * @param c the text field
     * @return the UI
     */
    public static ComponentUI createUI(JComponent c) {
        return new MotifTextFieldUI();
    }

    /**
     * Fetches the name used as a key to lookup properties through the
     * UIManager.  This is used as a prefix to all the standard
     * text properties.
     *
     * @return the name
     */
    protected String getPropertyPrefix() {
	return "TextField";
    }

    /**
     * Creates the controller supplied by the superclass, but
     * also extended to completely repaint when focus is 
     * gained or lost to repaint the focus rectangle.
     *
     * @param c the editor component
     * @return the controller
     */
  /*
    protected TextController createController(JTextComponent c) {
	return new MotifFieldController(c);
    }
    */

    /**
     * Creates a view based on an element.
     *
     * @param elem the element
     * @return the view
     */
    public View create(Element elem) {
	return new FieldView(elem);
    }

    /**
     * Creates a view based on an element.
     *
     * @param elem the element
     * @param p0 the starting offset
     * @param p1 the ending offset
     * @return the view
     */
    public View create(Element elem, int p0, int p1) {
	throw new Error("unsupported");
    }

    /**
     * Loads the given keymap with default settings
     * appropriate for the UI being created.  This
     * is implemented to do the superclass behavior 
     * as well as load the settings appropriate for
     * a single-line text editor.
     *
     * @param map the keymap to load
     */
    protected void loadDefaultKeymap(Keymap map) {
	super.loadDefaultKeymap(map);
	JTextComponent.loadKeymap(map, fieldBindings, 
					 getComponent().getActions());
    }

    /**
     * Paints the background of the field.
     *
     * @param g the graphics context
     */
    protected void paintBackground(Graphics g) {
	JTextField field = (JTextField) getComponent();
	if(field.isOpaque()) {
	    g.setColor(field.getBackground());
	    Dimension d = field.getSize();
	    g.fillRect(-field.getScrollOffset(), 0, d.width, d.height);
	}
    }

    /**
     * Default bindings multi-line editor keymaps implementing the Motif feel.
     */
    static final JTextComponent.KeyBinding[] fieldBindings = {
	new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
				       JTextField.notifyAction)
    };

}
