/*
 * @(#)SecurityPermission.java	1.11 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
 
package java.security;

import java.security.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * This class is for security permissions.  
 * A SecurityPermission contains a name but
 * no actions list; you either have the named permission 
 * or you don't.
 * <P>
 * Name is the name of a security configuration parameter. 
 * Currently the SecurityPermission object is used to guard access 
 * to the Identity object, and to the Security object.  The Identity
 * object is used to implement the digital signature identity database. 
 * <p>
 * To request permission to make changes in the Identity object, create a 
 * SecurityPermission with the name of the directive. 
 * Directives known by the base JDK are "set.public.key", "set.info", 
 * "add.certificate", "remove.certificate", "print".    
 * <P>
 * @see java.security.BasicPermission
 * @see java.security.Permission
 * @see java.security.Permissions
 * @see java.security.PermissionCollection
 * @see java.lang.SecurityManager
 *
 * @version 1.11 98/03/18
 *
 * @author Marianne Mueller
 * @author Roland Schemers
 */

public final class SecurityPermission extends BasicPermission {

    /** use serialVersionUID from JDK 1.2 for interoperability */
    private static final long serialVersionUID = 5236109936224050470L;

    /**
     * Creates a new SecurityPermission with the specified name.
     * The name is the symbolic name of the SecurityPermission. An asterisk 
     * may appear at the end of the name, following a ".", or by itself, to 
     * signify a wildcard match.
     *
     * @param name the name of the SecurityPermission
     */

    public SecurityPermission(String name)
    {
	super(name);
    }

    /**
     * Creates a new SecurityPermission object with the specified name.
     * The name is the symbolic name of the SecurityPermission, and the
     * actions String is currently unused and should be null.  This
     * constructor exists for use by the <code>Policy</code> object
     * to instantiate new Permission objects.
     *
     * @param name the name of the SecurityPermission
     * @param actions should be null.
     */

    public SecurityPermission(String name, String actions) 
    {
	super(name, actions);
    }
}
