/*
 * @(#)EncodedKeySpec.java	1.7 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package java.security.spec;

/**
 * This class represents a public or private key in encoded format.
 *
 * @author Jan Luehe
 *
 * @version 1.7, 98/03/18
 *
 * @see java.security.Key
 * @see java.security.KeyFactory
 * @see KeySpec
 * @see X509EncodedKeySpec
 * @see PKCS8EncodedKeySpec
 *
 * @since JDK1.2
 */

public abstract class EncodedKeySpec implements KeySpec {

    /**
     * Returns the encoded key.
     *
     * @return the encoded key.
     */
    public abstract byte[] getEncoded();

    /**
     * Returns the name of the encoding format associated with this
     * key specification.
     * 
     * <p>If the opaque representation of a key 
     * (see <a href = "java.security.Key.html">Key</a>) can be transformed
     * (see <a href = "java.security.KeyFactory.html">KeyFactory</a>) 
     * into this key specification (or a subclass of it),
     * <code>getFormat</code> called
     * on the opaque key returns the same value as the 
     * <code>getFormat</code> method
     * of this key specification.
     * 
     * @return a string representation of the encoding format.
     */
    public abstract String getFormat();
}
