/*
 * @(#)NVList.java	1.15 98/03/18
 *
 * Copyright 1996, 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/** 
 * NVList is a list of NamedValue objects.
 * The NVList class is used in the Dynamic Invocation Interface and
 * the Dynamic Skeleton Interface to describe arguments for a request,
 * as well as in the Context object to describe context
 * values. It maintains a modifiable list of NamedValues, each of
 * which consists of a string name, an Any value and an integer flag.
 * An NVList is a partially opaque structure. It may only be allocated
 * by a call to org.omg.CORBA.ORB.create_list() or create_operation_list().
 *
 * @see org.omg.CORBA.Request
 * @see org.omg.CORBA.ServerRequest
 * @see org.omg.CORBA.NamedValue
 * @see org.omg.CORBA.Context
 *
 * @version 1.15 ,03/18/98 
 * @since       JDK1.2
 */

public abstract class NVList {

  /**
   * A count of the number of NamedValues in the NVList.
   *
   * @return			number of NamedValues in the NVList.
   */

  public abstract int count();

  /**
   * Add a NamedValue with the given flags to the end of the list.
   * The flags can be any one of the argument passing modes:
   * ARG_IN.value, ARG_OUT.value or ARG_INOUT.value.
   *
   * @param flags		flags for the new NamedValue.
   * @return			the NamedValue created.
   */

  public abstract NamedValue add(int flags);

  /**
   * Add a NamedValue with the given name and flags to the end of the list.
   * The flags can be any one of the argument passing modes.
   *
   * @param item_name		the name for the new NamedValue.
   * @param flags		flags for the new NamedValue.
   * @return			the NamedValue created.
   */

  public abstract NamedValue add_item(String item_name, int flags);

  /**
   * Add a NamedValue with the given name, value and flags to the end of the list.
   * The flags can be any one of the argument passing modes.
   *
   * @param item_name		the name for the new NamedValue.
   * @param value		the Any value for the new NamedValue.
   * @param flags		flags for the new NamedValue.
   * @return			the NamedValue created.
   */

  public abstract NamedValue add_value(String item_name, Any val, int flags);

  /**
   * Return the NamedValue at the given index.
   *
   * @param index		the index of NamedValue desired.
   * @return			the NamedValue at the given index.
   * @exception org.omg.CORBA.Bounds raised if the index is greater than
   *				or equal to number of NamedValues.
   */

  public abstract NamedValue item(int index) throws org.omg.CORBA.Bounds;

  /**
   * Remove the NamedValue at the given index. Note that the indices
   * of all NamedValues following this one are shifted down by one.
   *
   * @param index		the index of NamedValue to be removed.
   * @exception org.omg.CORBA.Bounds raised if the index is greater than
   *				or equal to number of NamedValues.
   */

  public abstract void remove(int index) throws org.omg.CORBA.Bounds;

}
