/*
 * @(#)Object.java	1.34 98/03/18
 *
 * Copyright 1995-1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/**
 * This interface defines a CORBA Object reference.
 * <p>
 * A CORBA object reference is a handle for a particular
 * CORBA object implemented by a server. A CORBA object reference
 * identifies the same CORBA object each time the reference is used to invoke
 * a method on the object.
 * A CORBA object may have multiple, distinct object references.
 * <p>
 * The org.omg.CORBA.Object interface is the root of the inheritance hierarchy 
 * for all IDL interfaces mapped to Java, analogous to java.rmi.Remote 
 * for Java-RMI remote objects.
 * <p>
 * A CORBA object may be a local object (i.e. in the same VM as the client), 
 * in which case invocations may be directly serviced by the object instance,
 * and the object reference could be the actual instance of the object 
 * implementation class.
 * A CORBA object may be a remote object (i.e. in a different VM from the
 * client), in which case the object reference is a stub (proxy) which uses the
 * ORB machinery to make a remote invocation on the server where the object
 * implementation resides.
 * <p>
 * Default implementations of the methods in org.omg.CORBA.Object 
 * are provided in org.omg.CORBA.portable.ObjectImpl, which is the base class
 * for stubs and object implementations.
 * <p>
 * @see ObjectImpl
 */

public interface Object {
  
    /**
     * Check if this object supports the interface represented by the 
     * repositoryIdentifier string: 
     * i.e. check if this object is an instance of the given interface.
     * @param repositoryIdentifier the interface to check against
     * @return true if the object reference supports the interface
     */
    public boolean _is_a(String repositoryIdentifier);


    /**
     * Determines if the two object references are equivalent, so far as the
     * ORB can easily determine. If two objects are identical, they are
     * equivalent. Two distinct object references which in fact refer to 
     * the same object are also equivalent. However, ORBs are not required
     * to attempt determination of whether two distinct object references
     * refer to the same object, since such determination could be impractically
     * expensive.
     * @param other the other object reference to check equivalence with
     * @return true if this object reference is known to be equivalent to the
     *         other object reference passed in as parameter.
     *         A false only indicates that the two object references are 
     *         distinct, not necessarily that they reference distinct objects.
     */
    public boolean _is_equivalent(org.omg.CORBA.Object other);

    
   /**
    * Determines whether the server object for this object reference has been 
    * destroyed. 
    * @return true if the ORB knows authoritatively that the server object does 
    *         not exist, false otherwise
    */
    public boolean _non_existent();


   /**
    * Returns an ORB-internal identifier for this object reference which does
    * not change during the lifetime of the object reference and so
    * neither will any hash function of that identifier. The value returned
    * is not guaranteed to be unique; i.e. another object reference may have
    * the same hash value. If two object references hash differently,
    * then they are distinct object references (however both may still refer
    * to the same server CORBA object).
    * @param maximum specifies an upper bound on the hash value returned by 
    *            the ORB. 
    * @return ORB-internal hash identifier for object reference
    */
    public int _hash(int maximum);


   /**
    * Returns a duplicate of this CORBA object reference.
    * Note that the server object implementation is not involved in creating
    * the duplicate, and the implementation cannot distinguish whether
    * the original or a duplicate reference was used to make a request.
    * Note that in Java, since memory management is handled by the VM,  
    * _duplicate() is not very useful: it is included for compliance with
    * the CORBA APIs. _duplicate() may return this object reference itself.
    * @return a duplicate of this object reference
    */
    public org.omg.CORBA.Object _duplicate();


   /**
    * Indicates that the caller is done using this object reference, so
    * internal ORB resources associated with this object reference can be 
    * released. Note that the object implementation is not involved in
    * this operation, and other references to the same object are not affected.
    */
    public void _release();


    /**
     * Obtain an ImplementationDef for this object reference
     * @return ImplementationDef Object in Implementation Repository that 
     *         describes the implementation of this object
     */
     org.omg.CORBA.ImplementationDef _get_implementation();


    /**
     * Obtain an InterfaceDef for this object reference. The InterfaceDef
     * may be used to introspect on the methods, attributes and other
     * type information for this object reference.
     * @return InterfaceDef object in Interface Repository which provides
     *         type information about this object
     */
     org.omg.CORBA.InterfaceDef _get_interface();



     /** 
      * Create a Request instance for use in the Dynamic Invocation Interface.
      * @param operation The name of the operation to be invoked using the
      *		         Request instance.
      * @return the created Request instance
      */
     Request _request(String operation);



     /**
      * Create a Request instance for use in the Dynamic Invocation Interface.
      *
      * @param ctx			The context to be used.
      * @param operation		The name of the operation to be
      * 				invoked. 
      * @param arg_list		The arguments to the operation in the
      * 				form of an NVList.
      * @param result		A container for the result as a NamedValue.
      * @return			The created Request object.
      *
      * @see Request
      * @see NVList
      * @see NamedValue
      */
   
     Request _create_request(Context ctx, 
			     String operation, 
			     NVList arg_list,
			     NamedValue result);
  
     /**
      * Create a Request instance for use in the Dynamic Invocation Interface.
      *
      * @param ctx			The context to be used.
      * @param operation		The name of the operation to be
      * 				invoked. 
      * @param arg_list		The arguments to the operation in the
      * 				form of an NVList.
      * @param result		A container for the result as a NamedValue.
      * @param exclist		A list of possible exceptions the
      * 				operation can throw.
      * @param ctxlist		A list of context strings that need
      * 				to be resolved and sent with the
      * 				Request. 
      * @return			The created Request object.
      *
      * @see Request
      * @see NVList
      * @see NamedValue
      * @see ExceptionList
      * @see ContextList
      */
   
     Request _create_request(Context ctx, 
			     String operation, 
			     NVList arg_list,
			     NamedValue result, 
			     ExceptionList exclist, 
			     ContextList ctxlist);
   
}
