/*
 * @(#)ServerRequest.java	1.16 98/03/18
 *
 * Copyright 1996, 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package org.omg.CORBA;

/** 
 * The ServerRequest pseudo-object is the cornerstone of the ORB
 * Dynamic Skeleton Interface (DSI). It captures the explicit state of
 * a request for the DSI, analogous to the Request pseudo object in
 * the DII. <p>
 *
 * The ORB is responsible for creating this embodiment of a request,
 * and delivering it to a Dynamic Implementation Routine (DIR). This
 * is an instance of the DynamicImplementation interface. A dynamic
 * servant is created by implementing the DynamicImplementation
 * interface, which has a single invoke operation that accepts a
 * ServerRequest. <p>
 *
 * The ServerRequest interface defines operations to access the
 * operation name, the arguments and the context of the request, as
 * well as operations to set the response of the request either as a
 * result or an exception. <p>
 *
 * A subtlety with accessing the arguments of the request is that the
 * DIR needs to provide type information about the
 * expected arguments, since there is no compiled information about
 * these. This information is provided through an NVList, which is a
 * list of NamedValues. Each NamedValue contains an Any that has a
 * TypeCode representing the type of the corresponding argument. <p>
 *
 * Similarly, type information needs to be provided for the response,
 * for either the expected result or for an exception. <p>
 *
 * @see org.omg.CORBA.DynamicImplementation
 * @see org.omg.CORBA.NVList
 * @see org.omg.CORBA.NamedValue
 *
 * @version 1.16 03/18/98 
 */

public abstract class ServerRequest {

  /**
   * The op_name operation returns the name of the operation being
   * invoked; according to OMG IDL's rules, these names must be unique
   * among all operations supported by this object's "most-derived"
   * interface. Note that the operation names for getting and setting
   * attributes are _get_<attribute_name> and _set_<attribute_name>,
   * respectively. 
   *
   * @return			the name of the operation to be invoked
   */

  public abstract String op_name();

  /**
   * Operation parameters will be retrieved with params. They appear
   * in the NVList in the order in which they appear in the OMG IDL
   * specification (left to right). This holds the "in", "out" and
   * "inout" values. <p>
   *
   * The ServerRequest provides parameter values when the DIR invokes
   * the params() operation. The NVList provided by the DIR to the ORB
   * includes the TypeCodes (inside a NamedValue) for all parameters,
   * including out ones (their values are nulls at first), for the
   * operation. This allows the ORB to verify that the correct
   * parameter types have been provided before filling their values
   * in, but does not require it to do so. It also relieves the ORB of
   * all responsibility to consult the interface repository, promoting
   * high performance implementations. <p>
   *
   * In order to guarantee that the ORB could always verify parameter
   * lists, and to detect errors such as omitted parameters, Dynamic
   * Implementation Routines are always required to call params(),
   * even when the DIR believes that no parameters are used by the
   * operation. When the DIR believes no parameters are used by the
   * operation, it passes an empty NVList. <p>
   *
   * The params() and result() operations may be called only once, and
   * in that exact order. A violation results in the BAD_INV_ORDER 
   * exception being thrown.<p>
   *
   * @param params		the arguments of the operation, in the
   *				form of an NVList.
   * */

  public abstract void params(NVList params);
  
  /**
   * The result operation is used to specify any return
   * value for the call.  <p>
   *
   * The ServerRequest will not send a response to the invocation
   * until the DIR returns. If a return value is required, the
   * result() operation must be invoked to provide that value to the
   * ORB. Where no return value is required, this need not be invoked.
   * <p>
   * 
   * The params() and result() operations may be called only once, and
   * in that exact order. A violation results in the BAD_INV_ORDER 
   * exception being thrown.<p>
   *
   * @param a		the Any in which the result is stored.
   */

  public abstract void result(Any a); 
  
  /**
   * To report an exception, rather than provide return values, the
   * DIR provides the exception value inside an Any, and passes that
   * to exception(). <p>
   *
   * The exception() routine can be called only once, after params()
   * is called. It may not be called if result() has been called. A 
   * violation results in the BAD_INV_ORDER exception being thrown.<p>
   *
   * @param a			the Any containing the exception.
   */

  public abstract void except(Any a);
  
  /**
   * When the operation is not an attribute access, ctx will return
   * the context information defined in OMG IDL for operation (if any).
   * Otherwise, this context is empty. 
   *
   * @return			the context object that is to be used
   *				to resolve any context strings whose
   *				values need to be sent with the invocation.
   */

  public abstract Context ctx();
  
}
