/*
 * @(#)Delegate.java	1.15 98/03/18
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package org.omg.CORBA.portable;

import org.omg.CORBA.ImplementationDef;
import org.omg.CORBA.InterfaceDef;
import org.omg.CORBA.Request;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.NVList;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.TypeCode;

/** The Delegate abstract class specifies a portable API for 
 *  ORB-vendor-specific implementation of the org.omg.CORBA.Object methods. 
 *  Each stub (proxy) contains a delegate
 *  object, to which all org.omg.CORBA.Object methods are forwarded.
 *  This allows a stub generated by one vendor's ORB to work with the delegate
 *  from another vendor's ORB.
 *
 *  @see org.omg.CORBA.Object
 */

public abstract class Delegate {

  /** Return an ImplementationDef for the object reference provided.
   *  @param obj The object reference which delegated to this delegate, and
   *             whose ImplementationDef needs to be returned.
   *  @return the ImplementationDef 
   */
  public abstract ImplementationDef get_implementation( 
						org.omg.CORBA.Object obj);

  /** Return an InterfaceDef for the object reference provided.
   *  @param obj The object reference whose InterfaceDef needs to be returned
   *  @return the InterfaceDef 
   */
  public abstract InterfaceDef get_interface(org.omg.CORBA.Object obj);

  /** Return a duplicate of the object reference provided.
   *  @param obj The object reference whose duplicate needs to be returned
   *  @return the duplicate object reference 
   */
  public abstract org.omg.CORBA.Object duplicate(org.omg.CORBA.Object obj);

  /** Release resources associated with the object reference provided.
   *  @param obj The object reference whose resources need to be released
   */
  public abstract void release(org.omg.CORBA.Object obj);

  /**
   *  Check if the object reference is an instance of the given interface.
   *  @param obj The object reference to be checked.
   *  @param repository_id The repository identifier of the interface to check
			   against.
   *  @return true if the object reference supports the interface
   */
  public abstract boolean is_a(org.omg.CORBA.Object obj, String repository_id);

  /**
   *  Determines whether the server object for the object reference has been
   *  destroyed.
   *  @param obj The object reference which delegated to this delegate.
   *  @result true if the ORB knows authoritatively that the server object does
   *         not exist, false otherwise
   */
  public abstract boolean non_existent(org.omg.CORBA.Object obj);

  /** Determines if the two object references are equivalent.
   *  @param obj The object reference which delegated to this delegate.
   *  @param other The object reference to check equivalence against.
   */
  public abstract boolean is_equivalent(org.omg.CORBA.Object obj,
    					org.omg.CORBA.Object other);

  /**
   * Returns an ORB-internal identifier (hashcode) for this object reference.
   * @param obj The object reference which delegated to this delegate.
   * @param maximum specifies an upper bound on the hash value returned by
   *            the ORB.
   * @return ORB-internal hash identifier for object reference
   */
  public abstract int hash(org.omg.CORBA.Object obj, int max);

  /**
   * Create a Request instance for use in the Dynamic Invocation Interface.
   * @param obj The object reference which delegated to this delegate.
   * @param operation The name of the operation to be invoked using the
   *                  Request instance.
   * @return the created Request instance
   */
  public abstract Request request(org.omg.CORBA.Object obj, String operation);

  /**
   * Create a Request instance for use in the Dynamic Invocation Interface.
   *
   * @param obj The object reference which delegated to this delegate.
   * @param ctx                      The context to be used.
   * @param operation                The name of the operation to be
   *                                 invoked.
   * @param arg_list         The arguments to the operation in the
   *                                 form of an NVList.
   * @param result           A container for the result as a NamedValue.
   * @return                 The created Request object.
   *
   */
  public abstract Request create_request(org.omg.CORBA.Object obj,
				Context ctx,
				String operation,
				NVList arg_list,
				NamedValue result);

  /**
   * Create a Request instance for use in the Dynamic Invocation Interface.
   *
   * @param obj The object reference which delegated to this delegate.
   * @param ctx                      The context to be used.
   * @param operation                The name of the operation to be
   *                                 invoked.
   * @param arg_list         The arguments to the operation in the
   *                                 form of an NVList.
   * @param result           A container for the result as a NamedValue.
   * @param exclist          A list of possible exceptions the
   *                                 operation can throw.
   * @param ctxlist          A list of context strings that need
   *                                 to be resolved and sent with the
   *                                 Request.
   * @return                 The created Request object.
   */
  public abstract Request create_request(org.omg.CORBA.Object obj,
				Context ctx,
				String operation,
				NVList arg_list,
				NamedValue result,
				ExceptionList exclist,
				ContextList ctxlist);

  /**
   * Return the ORB that created this delegate.
   */
  public org.omg.CORBA.ORB orb(org.omg.CORBA.Object obj) {
      throw new org.omg.CORBA.NO_IMPLEMENT();
  }
}
